@@ -247,7 +247,7 @@ public class ThreadLocalFury extends AbstractThreadSafeFury {
 
   @Override
   public void setClassLoader(ClassLoader classLoader) {
-    setClassLoader(classLoader, StagingType.SOFT_STAGING);
+    setClassLoader(classLoader, StagingType.STRONG_STAGING);
   }
 
   @Override
