@@ -25,6 +25,7 @@ import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.atomic.AtomicInteger;
 import org.apache.fury.Fury;
+import org.apache.fury.meta.MetaCompressor;
 import org.apache.fury.serializer.Serializer;
 import org.apache.fury.serializer.TimeSerializers;
 import org.apache.fury.util.Preconditions;
@@ -49,9 +50,11 @@ public class Config implements Serializable {
   private final boolean requireClassRegistration;
   private final boolean suppressClassRegistrationWarnings;
   private final boolean registerGuavaTypes;
-  private final boolean shareMetaContext;
+  private final boolean metaShareEnabled;
+  private final boolean scopedMetaShareEnabled;
+  private final MetaCompressor metaCompressor;
   private final boolean asyncCompilationEnabled;
-  private final boolean deserializeUnexistedClass;
+  private final boolean deserializeNonexistentClass;
   private final boolean scalaOptimizationEnabled;
   private transient int configHash;
   private final boolean deserializeNonexistentEnumValueAsNull;
@@ -74,12 +77,14 @@ public class Config implements Serializable {
     compatibleMode = builder.compatibleMode;
     checkJdkClassSerializable = builder.checkJdkClassSerializable;
     defaultJDKStreamSerializerType = builder.defaultJDKStreamSerializerType;
-    shareMetaContext = builder.shareMetaContext;
-    deserializeUnexistedClass = builder.deserializeUnexistedClass;
-    if (deserializeUnexistedClass) {
+    metaShareEnabled = builder.metaShareEnabled;
+    scopedMetaShareEnabled = builder.scopedMetaShareEnabled;
+    metaCompressor = builder.metaCompressor;
+    deserializeNonexistentClass = builder.deserializeNonexistentClass;
+    if (deserializeNonexistentClass) {
       // Only in meta share mode or compatibleMode, fury knows how to deserialize
       // unexisted class by type info in data.
-      Preconditions.checkArgument(shareMetaContext || compatibleMode == CompatibleMode.COMPATIBLE);
+      Preconditions.checkArgument(metaShareEnabled || compatibleMode == CompatibleMode.COMPATIBLE);
     }
     asyncCompilationEnabled = builder.asyncCompilationEnabled;
     scalaOptimizationEnabled = builder.scalaOptimizationEnabled;
@@ -178,16 +183,32 @@ public class Config implements Serializable {
     return defaultJDKStreamSerializerType;
   }
 
-  public boolean shareMetaContext() {
-    return shareMetaContext;
+  public boolean isMetaShareEnabled() {
+    return metaShareEnabled;
+  }
+
+  /**
+   * Scoped meta share focuses on a single serialization process. Metadata created or identified
+   * during this process is exclusive to it and is not shared with by other serializations.
+   */
+  public boolean isScopedMetaShareEnabled() {
+    return scopedMetaShareEnabled;
+  }
+
+  /**
+   * Returns a {@link MetaCompressor} to compress class metadata such as field names and types. The
+   * returned {@link MetaCompressor} should be thread safe.
+   */
+  public MetaCompressor getMetaCompressor() {
+    return metaCompressor;
   }
 
   /**
    * Whether deserialize/skip data of un-existed class. If not enabled, an exception will be thrown
    * if class not exist.
    */
-  public boolean deserializeUnexistedClass() {
-    return deserializeUnexistedClass;
+  public boolean deserializeNonexistentClass() {
+    return deserializeNonexistentClass;
   }
 
   /**
@@ -235,9 +256,11 @@ public class Config implements Serializable {
         && requireClassRegistration == config.requireClassRegistration
         && suppressClassRegistrationWarnings == config.suppressClassRegistrationWarnings
         && registerGuavaTypes == config.registerGuavaTypes
-        && shareMetaContext == config.shareMetaContext
+        && metaShareEnabled == config.metaShareEnabled
+        && scopedMetaShareEnabled == config.scopedMetaShareEnabled
+        && Objects.equals(metaCompressor, config.metaCompressor)
         && asyncCompilationEnabled == config.asyncCompilationEnabled
-        && deserializeUnexistedClass == config.deserializeUnexistedClass
+        && deserializeNonexistentClass == config.deserializeNonexistentClass
         && scalaOptimizationEnabled == config.scalaOptimizationEnabled
         && language == config.language
         && compatibleMode == config.compatibleMode
@@ -265,9 +288,11 @@ public class Config implements Serializable {
         requireClassRegistration,
         suppressClassRegistrationWarnings,
         registerGuavaTypes,
-        shareMetaContext,
+        metaShareEnabled,
+        scopedMetaShareEnabled,
+        metaCompressor,
         asyncCompilationEnabled,
-        deserializeUnexistedClass,
+        deserializeNonexistentClass,
         scalaOptimizationEnabled);
   }
 
