@@ -27,6 +27,8 @@ import org.apache.fury.ThreadSafeFury;
 import org.apache.fury.logging.Logger;
 import org.apache.fury.logging.LoggerFactory;
 import org.apache.fury.memory.Platform;
+import org.apache.fury.meta.DeflaterMetaCompressor;
+import org.apache.fury.meta.MetaCompressor;
 import org.apache.fury.pool.ThreadPoolFury;
 import org.apache.fury.resolver.ClassResolver;
 import org.apache.fury.serializer.JavaSerializer;
@@ -68,14 +70,16 @@ public final class FuryBuilder {
   boolean checkJdkClassSerializable = true;
   Class<? extends Serializer> defaultJDKStreamSerializerType = ObjectStreamSerializer.class;
   boolean requireClassRegistration = true;
-  boolean shareMetaContext = false;
+  Boolean metaShareEnabled;
+  Boolean scopedMetaShareEnabled;
   boolean codeGenEnabled = true;
-  Boolean deserializeUnexistedClass;
+  Boolean deserializeNonexistentClass;
   boolean asyncCompilationEnabled = false;
   boolean registerGuavaTypes = true;
   boolean scalaOptimizationEnabled = false;
   boolean suppressClassRegistrationWarnings = true;
   boolean deserializeNonexistentEnumValueAsNull = false;
+  MetaCompressor metaCompressor = new DeflaterMetaCompressor();
 
   public FuryBuilder() {}
 
@@ -233,18 +237,40 @@ public final class FuryBuilder {
   }
 
   /** Whether to enable meta share mode. */
-  public FuryBuilder withMetaContextShare(boolean shareMetaContext) {
-    this.shareMetaContext = shareMetaContext;
+  public FuryBuilder withMetaShare(boolean shareMeta) {
+    this.metaShareEnabled = shareMeta;
+    if (!shareMeta) {
+      scopedMetaShareEnabled = false;
+    }
+    return this;
+  }
+
+  /**
+   * Scoped meta share focuses on a single serialization process. Metadata created or identified
+   * during this process is exclusive to it and is not shared with by other serializations.
+   */
+  public FuryBuilder withScopedMetaShare(boolean scoped) {
+    scopedMetaShareEnabled = scoped;
+    return this;
+  }
+
+  /**
+   * Set a compressor for meta compression. Note that the passed {@link MetaCompressor} should be
+   * thread-safe. By default, a `Deflater` based compressor {@link DeflaterMetaCompressor} will be
+   * used. Users can pass other compressor such as `zstd` for better compression rate.
+   */
+  public FuryBuilder withMetaCompressor(MetaCompressor metaCompressor) {
+    this.metaCompressor = MetaCompressor.checkMetaCompressor(metaCompressor);
     return this;
   }
 
   /**
    * Whether deserialize/skip data of un-existed class.
    *
-   * @see Config#deserializeUnexistedClass()
+   * @see Config#deserializeNonexistentClass()
    */
-  public FuryBuilder withDeserializeUnexistedClass(boolean deserializeUnexistedClass) {
-    this.deserializeUnexistedClass = deserializeUnexistedClass;
+  public FuryBuilder withDeserializeNonexistentClass(boolean deserializeNonexistentClass) {
+    this.deserializeNonexistentClass = deserializeNonexistentClass;
     return this;
   }
 
@@ -301,12 +327,31 @@ public final class FuryBuilder {
     }
     if (compatibleMode == CompatibleMode.COMPATIBLE) {
       checkClassVersion = false;
-      if (deserializeUnexistedClass == null) {
-        deserializeUnexistedClass = true;
+      if (deserializeNonexistentClass == null) {
+        deserializeNonexistentClass = true;
+      }
+      if (scopedMetaShareEnabled == null) {
+        if (metaShareEnabled == null) {
+          metaShareEnabled = true;
+          scopedMetaShareEnabled = true;
+        } else {
+          scopedMetaShareEnabled = false;
+        }
+      } else {
+        if (metaShareEnabled == null) {
+          metaShareEnabled = scopedMetaShareEnabled;
+        }
       }
     } else {
-      if (deserializeUnexistedClass == null) {
-        deserializeUnexistedClass = false;
+      if (deserializeNonexistentClass == null) {
+        deserializeNonexistentClass = false;
+      }
+      if (scopedMetaShareEnabled != null) {
+        LOG.warn("Scoped meta share is for CompatibleMode only, disable it for {}", compatibleMode);
+      }
+      scopedMetaShareEnabled = false;
+      if (metaShareEnabled == null) {
+        metaShareEnabled = false;
       }
     }
     if (!requireClassRegistration) {
