@@ -233,6 +233,14 @@ public final class MemoryBuffer {
     return heapMemory == null;
   }
 
+  /**
+   * Returns <tt>true</tt>, if the memory buffer is backed by heap memory and memory buffer can
+   * write to the whole memory region of underlying byte array.
+   */
+  public boolean isHeapFullyWriteable() {
+    return heapMemory != null && heapOffset == 0;
+  }
+
   /**
    * Get the heap byte array object.
    *
