@@ -27,9 +27,9 @@ import static org.apache.fury.type.TypeUtils.mapOf;
 
 import java.io.ObjectStreamClass;
 import java.io.Serializable;
+import java.lang.reflect.Array;
 import java.lang.reflect.Field;
 import java.util.ArrayList;
-import java.util.Collection;
 import java.util.Comparator;
 import java.util.HashMap;
 import java.util.List;
@@ -38,6 +38,7 @@ import java.util.Objects;
 import java.util.SortedMap;
 import org.apache.fury.Fury;
 import org.apache.fury.builder.MetaSharedCodecBuilder;
+import org.apache.fury.collection.Tuple2;
 import org.apache.fury.config.CompatibleMode;
 import org.apache.fury.config.FuryBuilder;
 import org.apache.fury.logging.Logger;
@@ -48,9 +49,11 @@ import org.apache.fury.reflect.ReflectionUtils;
 import org.apache.fury.reflect.TypeRef;
 import org.apache.fury.resolver.ClassResolver;
 import org.apache.fury.serializer.CompatibleSerializer;
+import org.apache.fury.serializer.NonexistentClass;
 import org.apache.fury.type.Descriptor;
 import org.apache.fury.type.FinalObjectTypeStub;
 import org.apache.fury.type.GenericType;
+import org.apache.fury.type.TypeUtils;
 import org.apache.fury.util.Preconditions;
 
 /**
@@ -67,7 +70,7 @@ import org.apache.fury.util.Preconditions;
  * @see MetaSharedCodecBuilder
  * @see CompatibleMode#COMPATIBLE
  * @see CompatibleSerializer
- * @see FuryBuilder#withMetaContextShare
+ * @see FuryBuilder#withMetaShare
  * @see ReflectionUtils#getFieldOffset
  */
 public class ClassDef implements Serializable {
@@ -75,7 +78,8 @@ public class ClassDef implements Serializable {
 
   static final int SCHEMA_COMPATIBLE_FLAG = 0b10000;
   public static final int SIZE_TWO_BYTES_FLAG = 0b100000;
-  static final int EXT_FLAG = 0b1000000;
+  static final int OBJECT_TYPE_FLAG = 0b1000000;
+  static final int COMPRESSION_FLAG = 0b10000000;
   // TODO use field offset to sort field, which will hit l1-cache more. Since
   // `objectFieldOffset` is not part of jvm-specification, it may change between different jdk
   // vendor. But the deserialization peer use the class definition to create deserializer, it's OK
@@ -100,9 +104,9 @@ public class ClassDef implements Serializable {
         }
       };
 
-  private final String className;
+  private final ClassSpec classSpec;
   private final List<FieldInfo> fieldsInfo;
-  private final Map<String, String> extMeta;
+  private final boolean isObjectType;
   // Unique id for class def. If class def are same between processes, then the id will
   // be same too.
   private final long id;
@@ -110,14 +114,14 @@ public class ClassDef implements Serializable {
   private transient List<Descriptor> descriptors;
 
   ClassDef(
-      String className,
+      ClassSpec classSpec,
       List<FieldInfo> fieldsInfo,
-      Map<String, String> extMeta,
+      boolean isObjectType,
       long id,
       byte[] encoded) {
-    this.className = className;
+    this.classSpec = classSpec;
     this.fieldsInfo = fieldsInfo;
-    this.extMeta = extMeta;
+    this.isObjectType = isObjectType;
     this.id = id;
     this.encoded = encoded;
   }
@@ -128,7 +132,11 @@ public class ClassDef implements Serializable {
    * @see Class#getName()
    */
   public String getClassName() {
-    return className;
+    return classSpec.entireClassName;
+  }
+
+  public ClassSpec getClassSpec() {
+    return classSpec;
   }
 
   /** Contain all fields info including all parent classes. */
@@ -137,8 +145,8 @@ public class ClassDef implements Serializable {
   }
 
   /** Returns ext meta for the class. */
-  public Map<String, String> getExtMeta() {
-    return extMeta;
+  public boolean isObjectType() {
+    return isObjectType;
   }
 
   /**
@@ -162,26 +170,26 @@ public class ClassDef implements Serializable {
       return false;
     }
     ClassDef classDef = (ClassDef) o;
-    return Objects.equals(className, classDef.className)
+    return Objects.equals(classSpec.entireClassName, classDef.classSpec.entireClassName)
         && Objects.equals(fieldsInfo, classDef.fieldsInfo)
-        && Objects.equals(extMeta, classDef.extMeta);
+        && Objects.equals(id, classDef.id);
   }
 
   @Override
   public int hashCode() {
-    return Objects.hash(className, fieldsInfo, extMeta);
+    return Objects.hash(classSpec.entireClassName, fieldsInfo, id);
   }
 
   @Override
   public String toString() {
     return "ClassDef{"
         + "className='"
-        + className
+        + classSpec.entireClassName
         + '\''
         + ", fieldsInfo="
         + fieldsInfo
-        + ", extMeta="
-        + extMeta
+        + ", isObjectType="
+        + isObjectType
         + ", id="
         + id
         + '}';
@@ -189,7 +197,7 @@ public class ClassDef implements Serializable {
 
   /** Write class definition to buffer. */
   public void writeClassDef(MemoryBuffer buffer) {
-    buffer.writeBytes(encoded);
+    buffer.writeBytes(encoded, 0, encoded.length);
   }
 
   /** Read class definition from buffer. */
@@ -332,7 +340,7 @@ public class ClassDef implements Serializable {
       this.isMonomorphic = isMonomorphic;
     }
 
-    private final boolean isMonomorphic;
+    protected final boolean isMonomorphic;
 
     public boolean isMonomorphic() {
       return isMonomorphic;
@@ -364,31 +372,47 @@ public class ClassDef implements Serializable {
       return Objects.hash(isMonomorphic);
     }
 
-    public void write(MemoryBuffer buffer) {
+    /** Write field type info. */
+    public void write(MemoryBuffer buffer, boolean writeMonomorphicFlag) {
       byte header = (byte) (isMonomorphic ? 1 : 0);
-      if (this instanceof RegisteredFieldType) {
-        short classId = ((RegisteredFieldType) this).getClassId();
-        buffer.writeVarUint32Small7(((3 + classId) << 1) | header);
-      } else if (this instanceof CollectionFieldType) {
-        buffer.writeVarUint32Small7((2 << 1) | header);
-        ((CollectionFieldType) this).elementType.write(buffer);
-      } else if (this instanceof MapFieldType) {
-        buffer.writeVarUint32Small7((1 << 1) | header);
-        MapFieldType mapFieldType = (MapFieldType) this;
-        mapFieldType.keyType.write(buffer);
-        mapFieldType.valueType.write(buffer);
+      if (this instanceof ClassDef.RegisteredFieldType) {
+        short classId = ((ClassDef.RegisteredFieldType) this).getClassId();
+        buffer.writeVarUint32Small7(
+            writeMonomorphicFlag ? ((5 + classId) << 1) | header : 5 + classId);
+      } else if (this instanceof ClassDef.EnumFieldType) {
+        buffer.writeVarUint32Small7(writeMonomorphicFlag ? ((4) << 1) | header : 4);
+      } else if (this instanceof ClassDef.ArrayFieldType) {
+        ClassDef.ArrayFieldType arrayFieldType = (ClassDef.ArrayFieldType) this;
+        buffer.writeVarUint32Small7(writeMonomorphicFlag ? ((3) << 1) | header : 3);
+        buffer.writeVarUint32Small7(arrayFieldType.getDimensions());
+        (arrayFieldType).getComponentType().write(buffer);
+      } else if (this instanceof ClassDef.CollectionFieldType) {
+        buffer.writeVarUint32Small7(writeMonomorphicFlag ? ((2) << 1) | header : 2);
+        // TODO remove it when new collection deserialization jit finished.
+        ((ClassDef.CollectionFieldType) this).getElementType().write(buffer);
+      } else if (this instanceof ClassDef.MapFieldType) {
+        buffer.writeVarUint32Small7(writeMonomorphicFlag ? ((1) << 1) | header : 1);
+        // TODO remove it when new map deserialization jit finished.
+        ClassDef.MapFieldType mapFieldType = (ClassDef.MapFieldType) this;
+        mapFieldType.getKeyType().write(buffer);
+        mapFieldType.getValueType().write(buffer);
       } else {
-        Preconditions.checkArgument(this instanceof ObjectFieldType);
-        buffer.writeVarUint32Small7(header);
+        Preconditions.checkArgument(this instanceof ClassDef.ObjectFieldType);
+        buffer.writeVarUint32Small7(writeMonomorphicFlag ? header : 0);
       }
     }
 
+    public void write(MemoryBuffer buffer) {
+      write(buffer, true);
+    }
+
     public static FieldType read(MemoryBuffer buffer) {
       int header = buffer.readVarUint32Small7();
       boolean isMonomorphic = (header & 0b1) != 0;
       return read(buffer, isMonomorphic, header >>> 1);
     }
 
+    /** Read field type info. */
     public static FieldType read(MemoryBuffer buffer, boolean isFinal, int typeId) {
       if (typeId == 0) {
         return new ObjectFieldType(isFinal);
@@ -396,8 +420,13 @@ public class ClassDef implements Serializable {
         return new MapFieldType(isFinal, read(buffer), read(buffer));
       } else if (typeId == 2) {
         return new CollectionFieldType(isFinal, read(buffer));
+      } else if (typeId == 3) {
+        int dims = buffer.readVarUint32Small7();
+        return new ArrayFieldType(isFinal, read(buffer), dims);
+      } else if (typeId == 4) {
+        return EnumFieldType.getInstance();
       } else {
-        return new RegisteredFieldType(isFinal, (short) (typeId - 3));
+        return new RegisteredFieldType(isFinal, (short) (typeId - 5));
       }
     }
   }
@@ -571,6 +600,89 @@ public class ClassDef implements Serializable {
     }
   }
 
+  public static class EnumFieldType extends FieldType {
+    private static final EnumFieldType INSTANCE = new EnumFieldType();
+
+    private EnumFieldType() {
+      super(true);
+    }
+
+    @Override
+    public TypeRef<?> toTypeToken(ClassResolver classResolver) {
+      return TypeRef.of(NonexistentClass.NonexistentEnum.class);
+    }
+
+    public static EnumFieldType getInstance() {
+      return INSTANCE;
+    }
+  }
+
+  public static class ArrayFieldType extends FieldType {
+    private final FieldType componentType;
+    private final int dimensions;
+
+    public ArrayFieldType(boolean isMonomorphic, FieldType componentType, int dimensions) {
+      super(isMonomorphic);
+      this.componentType = componentType;
+      this.dimensions = dimensions;
+    }
+
+    @Override
+    public TypeRef<?> toTypeToken(ClassResolver classResolver) {
+      TypeRef<?> componentTypeRef = componentType.toTypeToken(classResolver);
+      Class<?> componentRawType = componentTypeRef.getRawType();
+      if (NonexistentClass.class.isAssignableFrom(componentRawType)) {
+        return TypeRef.of(
+            // We embed `isMonomorphic` flag in ObjectArraySerializer, so this flag can be ignored
+            // here.
+            NonexistentClass.getNonexistentClass(
+                componentType instanceof EnumFieldType, dimensions, true));
+      } else {
+        return TypeRef.of(Array.newInstance(componentRawType, new int[dimensions]).getClass());
+      }
+    }
+
+    public int getDimensions() {
+      return dimensions;
+    }
+
+    public FieldType getComponentType() {
+      return componentType;
+    }
+
+    @Override
+    public boolean equals(Object o) {
+      if (this == o) {
+        return true;
+      }
+      if (o == null || getClass() != o.getClass()) {
+        return false;
+      }
+      if (!super.equals(o)) {
+        return false;
+      }
+      ArrayFieldType that = (ArrayFieldType) o;
+      return dimensions == that.dimensions && Objects.equals(componentType, that.componentType);
+    }
+
+    @Override
+    public int hashCode() {
+      return Objects.hash(super.hashCode(), componentType, dimensions);
+    }
+
+    @Override
+    public String toString() {
+      return "ArrayFieldType{"
+          + "componentType="
+          + componentType
+          + ", dimensions="
+          + dimensions
+          + ", isMonomorphic="
+          + isMonomorphic
+          + '}';
+    }
+  }
+
   /** Class for field type which isn't registered and not collection/map type too. */
   public static class ObjectFieldType extends FieldType {
 
@@ -597,21 +709,26 @@ public class ClassDef implements Serializable {
   /** Build field type from generics, nested generics will be extracted too. */
   static FieldType buildFieldType(ClassResolver classResolver, Field field) {
     Preconditions.checkNotNull(field);
-    Class<?> rawType = field.getType();
-    boolean isFinal = classResolver.isMonomorphic(rawType);
-    if (Collection.class.isAssignableFrom(rawType)) {
-      GenericType genericType = GenericType.build(field.getGenericType());
+    GenericType genericType = GenericType.build(field.getGenericType());
+    return buildFieldType(classResolver, genericType);
+  }
+
+  /** Build field type from generics, nested generics will be extracted too. */
+  private static FieldType buildFieldType(ClassResolver classResolver, GenericType genericType) {
+    Preconditions.checkNotNull(genericType);
+    Class<?> rawType = genericType.getCls();
+    boolean isMonomorphic = genericType.isMonomorphic();
+    if (COLLECTION_TYPE.isSupertypeOf(genericType.getTypeRef())) {
       return new CollectionFieldType(
-          isFinal,
+          isMonomorphic,
           buildFieldType(
               classResolver,
               genericType.getTypeParameter0() == null
                   ? GenericType.build(Object.class)
                   : genericType.getTypeParameter0()));
-    } else if (Map.class.isAssignableFrom(rawType)) {
-      GenericType genericType = GenericType.build(field.getGenericType());
+    } else if (MAP_TYPE.isSupertypeOf(genericType.getTypeRef())) {
       return new MapFieldType(
-          isFinal,
+          isMonomorphic,
           buildFieldType(
               classResolver,
               genericType.getTypeParameter0() == null
@@ -625,44 +742,17 @@ public class ClassDef implements Serializable {
     } else {
       Short classId = classResolver.getRegisteredClassId(rawType);
       if (classId != null && classId != ClassResolver.NO_CLASS_ID) {
-        return new RegisteredFieldType(isFinal, classId);
+        return new RegisteredFieldType(isMonomorphic, classId);
       } else {
-        return new ObjectFieldType(isFinal);
-      }
-    }
-  }
-
-  /** Build field type from generics, nested generics will be extracted too. */
-  private static FieldType buildFieldType(ClassResolver classResolver, GenericType genericType) {
-    Preconditions.checkNotNull(genericType);
-    boolean isFinal = genericType.isMonomorphic();
-    if (COLLECTION_TYPE.isSupertypeOf(genericType.getTypeRef())) {
-      return new CollectionFieldType(
-          isFinal,
-          buildFieldType(
-              classResolver,
-              genericType.getTypeParameter0() == null
-                  ? GenericType.build(Object.class)
-                  : genericType.getTypeParameter0()));
-    } else if (MAP_TYPE.isSupertypeOf(genericType.getTypeRef())) {
-      return new MapFieldType(
-          isFinal,
-          buildFieldType(
-              classResolver,
-              genericType.getTypeParameter0() == null
-                  ? GenericType.build(Object.class)
-                  : genericType.getTypeParameter0()),
-          buildFieldType(
-              classResolver,
-              genericType.getTypeParameter1() == null
-                  ? GenericType.build(Object.class)
-                  : genericType.getTypeParameter1()));
-    } else {
-      Short classId = classResolver.getRegisteredClassId(genericType.getCls());
-      if (classId != null && classId != ClassResolver.NO_CLASS_ID) {
-        return new RegisteredFieldType(isFinal, classId);
-      } else {
-        return new ObjectFieldType(isFinal);
+        if (rawType.isEnum()) {
+          return EnumFieldType.getInstance();
+        }
+        if (rawType.isArray()) {
+          Tuple2<Class<?>, Integer> info = TypeUtils.getArrayComponentInfo(rawType);
+          return new ArrayFieldType(
+              isMonomorphic, buildFieldType(classResolver, GenericType.build(info.f0)), info.f1);
+        }
+        return new ObjectFieldType(isMonomorphic);
       }
     }
   }
@@ -673,17 +763,17 @@ public class ClassDef implements Serializable {
 
   public static ClassDef buildClassDef(Fury fury, Class<?> cls, boolean resolveParent) {
     return ClassDefEncoder.buildClassDef(
-        fury.getClassResolver(), cls, buildFields(fury, cls, resolveParent), new HashMap<>());
+        fury.getClassResolver(), cls, buildFields(fury, cls, resolveParent), true);
   }
 
   /** Build class definition from fields of class. */
   public static ClassDef buildClassDef(
       ClassResolver classResolver, Class<?> type, List<Field> fields) {
-    return buildClassDef(classResolver, type, fields, new HashMap<>());
+    return buildClassDef(classResolver, type, fields, true);
   }
 
   public static ClassDef buildClassDef(
-      ClassResolver classResolver, Class<?> type, List<Field> fields, Map<String, String> extMeta) {
-    return ClassDefEncoder.buildClassDef(classResolver, type, fields, extMeta);
+      ClassResolver classResolver, Class<?> type, List<Field> fields, boolean isObjectType) {
+    return ClassDefEncoder.buildClassDef(classResolver, type, fields, isObjectType);
   }
 }
