@@ -34,6 +34,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.LinkedHashSet;
 import java.util.List;
@@ -506,7 +507,10 @@ public class ReflectionUtils {
     if (targetType.isArray()) {
       return isMonomorphic(targetType.getComponentType());
     }
-    return Modifier.isFinal(targetType.getModifiers());
+    // Although enum itself can be non-final, but it's subclass can only be anonymous
+    // inner class, which is final, and we serialize enum by value, which already
+    // identify the actual class, so we can take enum as monomorphic.
+    return targetType.isEnum() || Modifier.isFinal(targetType.getModifiers());
   }
 
   public static TypeRef getPublicSuperType(TypeRef typeRef) {
@@ -554,17 +558,22 @@ public class ReflectionUtils {
   }
 
   /**
-   * Returns the canonical name of the underlying class as defined by <cite>The Java Language
+   * Returns the literal name of the underlying class as defined by <cite>The Java Language
    * Specification</cite>. Throw {@link IllegalArgumentException} if the underlying class does not
    * have a canonical name(i.e., if it is a local or anonymous class or an array whose component
    * type does not have a canonical name).
    *
    * @throws IllegalArgumentException if the canonical name of the underlying class doesn't exist.
    */
-  public static String getCanonicalName(Class<?> cls) {
+  public static String getLiteralName(Class<?> cls) {
     String canonicalName = cls.getCanonicalName();
     org.apache.fury.util.Preconditions.checkArgument(
         canonicalName != null, "Class %s doesn't have canonical name", cls);
+    if (canonicalName.endsWith(".")) {
+      // qualifier name of scala object type will ends with `.`
+      String name = cls.getName();
+      canonicalName = name.substring(0, name.length() - 1).replace("$", ".") + "$";
+    }
     return canonicalName;
   }
 
@@ -688,8 +697,8 @@ public class ReflectionUtils {
       Object o2) {
 
     for (String commonField : commonFieldsInfo.f0) {
-      Field field1 = commonFieldsInfo.f1.get(commonField);
-      Field field2 = commonFieldsInfo.f2.get(commonField);
+      Field field1 = Objects.requireNonNull(commonFieldsInfo.f1.get(commonField));
+      Field field2 = Objects.requireNonNull(commonFieldsInfo.f2.get(commonField));
       FieldAccessor accessor1 = FieldAccessor.createAccessor(field1);
       FieldAccessor accessor2 = FieldAccessor.createAccessor(field2);
       Object f1 = accessor1.get(o1);
@@ -761,15 +770,13 @@ public class ReflectionUtils {
             .collect(
                 Collectors.toMap(
                     // don't use `getGenericType` since janino doesn't support generics.
-                    f -> f.getDeclaringClass().getSimpleName() + f.getType() + f.getName(),
+                    f -> f.getDeclaringClass().getSimpleName() + f.getName(),
                     f -> f));
     Map<String, Field> fieldMap2 =
         fields2.stream()
             .collect(
-                Collectors.toMap(
-                    f -> f.getDeclaringClass().getSimpleName() + f.getType() + f.getName(),
-                    f -> f));
-    Set<String> commonFields = fieldMap1.keySet();
+                Collectors.toMap(f -> f.getDeclaringClass().getSimpleName() + f.getName(), f -> f));
+    Set<String> commonFields = new HashSet<>(fieldMap1.keySet());
     commonFields.retainAll(fieldMap2.keySet());
     return Tuple3.of(commonFields, fieldMap1, fieldMap2);
   }
