@@ -19,17 +19,17 @@
 
 package org.apache.fury.resolver;
 
+import static org.apache.fury.meta.Encoders.GENERIC_ENCODER;
 import static org.apache.fury.meta.Encoders.PACKAGE_ENCODER;
-import static org.apache.fury.meta.Encoders.TYPE_NAME_ENCODER;
 
 import org.apache.fury.collection.Tuple2;
 import org.apache.fury.config.Language;
+import org.apache.fury.meta.ClassDef;
+import org.apache.fury.meta.Encoders;
 import org.apache.fury.meta.MetaString.Encoding;
 import org.apache.fury.reflect.ReflectionUtils;
 import org.apache.fury.serializer.Serializer;
-import org.apache.fury.type.TypeUtils;
 import org.apache.fury.util.Preconditions;
-import org.apache.fury.util.StringUtils;
 import org.apache.fury.util.function.Functions;
 
 /**
@@ -37,9 +37,6 @@ import org.apache.fury.util.function.Functions;
  * serialization.
  */
 public class ClassInfo {
-  static final String ARRAY_PREFIX = "1";
-  static final String ENUM_PREFIX = "2";
-
   final Class<?> cls;
   final MetaStringBytes fullClassNameBytes;
   final MetaStringBytes packageNameBytes;
@@ -50,6 +47,8 @@ public class ClassInfo {
   // use primitive to avoid boxing
   // class id must be less than Integer.MAX_VALUE/2 since we use bit 0 as class id flag.
   short classId;
+  ClassDef classDef;
+  boolean needToWriteClassDef;
 
   ClassInfo(
       Class<?> cls,
@@ -81,40 +80,26 @@ public class ClassInfo {
       short classId) {
     this.cls = cls;
     this.serializer = serializer;
+    needToWriteClassDef = serializer != null && classResolver.needToWriteClassDef(serializer);
     MetaStringResolver metaStringResolver = classResolver.getMetaStringResolver();
     if (cls != null && classResolver.getFury().getLanguage() != Language.JAVA) {
       this.fullClassNameBytes =
           metaStringResolver.getOrCreateMetaStringBytes(
-              PACKAGE_ENCODER.encode(cls.getName(), Encoding.UTF_8));
+              GENERIC_ENCODER.encode(cls.getName(), Encoding.UTF_8));
     } else {
       this.fullClassNameBytes = null;
     }
+    // When `classId == ClassResolver.REPLACE_STUB_ID` was established,
+    // means only classes are serialized, not the instance. If we
+    // serialize such class only, we need to write classname bytes.
     if (cls != null
         && (classId == ClassResolver.NO_CLASS_ID || classId == ClassResolver.REPLACE_STUB_ID)) {
       // REPLACE_STUB_ID for write replace class in `ClassSerializer`.
-      String packageName = ReflectionUtils.getPackage(cls);
-      String className = ReflectionUtils.getClassNameWithoutPackage(cls);
-      if (cls.isArray()) {
-        Tuple2<Class<?>, Integer> componentInfo = TypeUtils.getArrayComponentInfo(cls);
-        Class<?> ctype = componentInfo.f0;
-        if (!ctype.isPrimitive()) { // primitive array has special format like [[[III.
-          String componentName = ctype.getName();
-          packageName = ReflectionUtils.getPackage(componentName);
-          String componentSimpleName = ReflectionUtils.getClassNameWithoutPackage(componentName);
-          String prefix = StringUtils.repeat(ARRAY_PREFIX, componentInfo.f1);
-          if (ctype.isEnum()) {
-            className = prefix + ENUM_PREFIX + componentSimpleName;
-          } else {
-            className = prefix + componentSimpleName;
-          }
-        }
-      } else if (cls.isEnum()) {
-        className = ENUM_PREFIX + className;
-      }
+      Tuple2<String, String> tuple2 = Encoders.encodePkgAndClass(cls);
       this.packageNameBytes =
-          metaStringResolver.getOrCreateMetaStringBytes(PACKAGE_ENCODER.encode(packageName));
+          metaStringResolver.getOrCreateMetaStringBytes(Encoders.encodePackage(tuple2.f0));
       this.classNameBytes =
-          metaStringResolver.getOrCreateMetaStringBytes(TYPE_NAME_ENCODER.encode(className));
+          metaStringResolver.getOrCreateMetaStringBytes(Encoders.encodeTypeName(tuple2.f1));
     } else {
       this.packageNameBytes = null;
       this.classNameBytes = null;
@@ -163,6 +148,11 @@ public class ClassInfo {
     return (Serializer<T>) serializer;
   }
 
+  void setSerializer(ClassResolver resolver, Serializer<?> serializer) {
+    this.serializer = serializer;
+    needToWriteClassDef = serializer != null && resolver.needToWriteClassDef(serializer);
+  }
+
   @Override
   public String toString() {
     return "ClassInfo{"
