@@ -19,7 +19,6 @@
 
 package org.apache.fury.resolver;
 
-import static org.apache.fury.collection.Collections.ofHashMap;
 import static org.apache.fury.meta.ClassDef.SIZE_TWO_BYTES_FLAG;
 import static org.apache.fury.meta.Encoders.PACKAGE_DECODER;
 import static org.apache.fury.meta.Encoders.PACKAGE_ENCODER;
@@ -88,6 +87,8 @@ import org.apache.fury.codegen.Expression.Invoke;
 import org.apache.fury.codegen.Expression.Literal;
 import org.apache.fury.collection.IdentityMap;
 import org.apache.fury.collection.IdentityObjectIntMap;
+import org.apache.fury.collection.LongMap;
+import org.apache.fury.collection.ObjectArray;
 import org.apache.fury.collection.ObjectMap;
 import org.apache.fury.collection.Tuple2;
 import org.apache.fury.config.CompatibleMode;
@@ -98,12 +99,12 @@ import org.apache.fury.logging.LoggerFactory;
 import org.apache.fury.memory.MemoryBuffer;
 import org.apache.fury.memory.Platform;
 import org.apache.fury.meta.ClassDef;
+import org.apache.fury.meta.ClassSpec;
+import org.apache.fury.meta.Encoders;
 import org.apache.fury.meta.MetaString;
 import org.apache.fury.reflect.ReflectionUtils;
 import org.apache.fury.reflect.TypeRef;
 import org.apache.fury.serializer.ArraySerializers;
-import org.apache.fury.serializer.ArraySerializers.UnexistedArrayClassSerializer;
-import org.apache.fury.serializer.ArraySerializers.UnexistedEnumArrayClassSerializer;
 import org.apache.fury.serializer.BufferSerializers;
 import org.apache.fury.serializer.CodegenSerializer.LazyInitBeanSerializer;
 import org.apache.fury.serializer.CompatibleSerializer;
@@ -114,6 +115,11 @@ import org.apache.fury.serializer.JdkProxySerializer;
 import org.apache.fury.serializer.LambdaSerializer;
 import org.apache.fury.serializer.LocaleSerializer;
 import org.apache.fury.serializer.MetaSharedSerializer;
+import org.apache.fury.serializer.NonexistentClass;
+import org.apache.fury.serializer.NonexistentClass.NonexistentMetaShared;
+import org.apache.fury.serializer.NonexistentClass.NonexistentSkip;
+import org.apache.fury.serializer.NonexistentClassSerializers;
+import org.apache.fury.serializer.NonexistentClassSerializers.NonexistentClassSerializer;
 import org.apache.fury.serializer.ObjectSerializer;
 import org.apache.fury.serializer.OptionalSerializers;
 import org.apache.fury.serializer.PrimitiveSerializers;
@@ -124,13 +130,6 @@ import org.apache.fury.serializer.Serializers;
 import org.apache.fury.serializer.StringSerializer;
 import org.apache.fury.serializer.StructSerializer;
 import org.apache.fury.serializer.TimeSerializers;
-import org.apache.fury.serializer.UnexistedClassSerializers.UnexistedArrayClass;
-import org.apache.fury.serializer.UnexistedClassSerializers.UnexistedClassSerializer;
-import org.apache.fury.serializer.UnexistedClassSerializers.UnexistedEnumArrayClass;
-import org.apache.fury.serializer.UnexistedClassSerializers.UnexistedEnumClassSerializer;
-import org.apache.fury.serializer.UnexistedClassSerializers.UnexistedMetaSharedClass;
-import org.apache.fury.serializer.UnexistedClassSerializers.UnexistedSkipClass;
-import org.apache.fury.serializer.UnexistedClassSerializers.UnexistedSkipEnumClass;
 import org.apache.fury.serializer.collection.ChildContainerSerializers;
 import org.apache.fury.serializer.collection.CollectionSerializer;
 import org.apache.fury.serializer.collection.CollectionSerializers;
@@ -150,7 +149,6 @@ import org.apache.fury.type.ScalaTypes;
 import org.apache.fury.type.TypeUtils;
 import org.apache.fury.util.GraalvmSupport;
 import org.apache.fury.util.Preconditions;
-import org.apache.fury.util.StringUtils;
 import org.apache.fury.util.function.Functions;
 
 /**
@@ -207,7 +205,9 @@ public class ClassResolver {
   private static final float loadFactor = 0.25f;
   private static final float furyMapLoadFactor = 0.25f;
   private static final int estimatedNumRegistered = 150;
-  private static final String META_SHARE_FIELDS_INFO_KEY = "shareFieldsInfo";
+  private static final String SET_META__CONTEXT_MSG =
+      "Meta context must be set before serialization, "
+          + "please set meta context by SerializationContext.setMetaContext";
   private static final ClassInfo NIL_CLASS_INFO =
       new ClassInfo(null, null, null, null, false, null, null, ClassResolver.NO_CLASS_ID);
 
@@ -246,7 +246,7 @@ public class ClassResolver {
     // ex. A->field1: B, B.field1: A
     private final Set<Class<?>> getClassCtx = new HashSet<>();
     private final Map<Class<?>, FieldResolver> fieldResolverMap = new HashMap<>();
-    private final Map<Long, Tuple2<ClassDef, ClassInfo>> classIdToDef = new HashMap<>();
+    private final LongMap<Tuple2<ClassDef, ClassInfo>> classIdToDef = new LongMap<>();
     private final Map<Class<?>, ClassDef> currentLayerClassDef = new HashMap<>();
     // TODO(chaokunyang) Better to  use soft reference, see ObjectStreamClass.
     private final ConcurrentHashMap<Tuple2<Class<?>, Boolean>, SortedMap<Field, Descriptor>>
@@ -260,7 +260,7 @@ public class ClassResolver {
     this.fury = fury;
     metaStringResolver = fury.getMetaStringResolver();
     classInfoCache = NIL_CLASS_INFO;
-    metaContextShareEnabled = fury.getConfig().shareMetaContext();
+    metaContextShareEnabled = fury.getConfig().isMetaShareEnabled();
     extRegistry = new ExtRegistry();
     extRegistry.objectGenericType = buildGenericType(OBJECT_TYPE);
     shimDispatcher = new ShimDispatcher(fury);
@@ -337,18 +337,18 @@ public class ClassResolver {
     if (fury.getConfig().registerGuavaTypes()) {
       GuavaCollectionSerializers.registerDefaultSerializers(fury);
     }
-    if (fury.getConfig().deserializeUnexistedClass()) {
+    if (fury.getConfig().deserializeNonexistentClass()) {
       if (metaContextShareEnabled) {
         addDefaultSerializer(
-            UnexistedMetaSharedClass.class, new UnexistedClassSerializer(fury, null));
+            NonexistentMetaShared.class, new NonexistentClassSerializer(fury, null));
         // Those class id must be known in advance, here is two bytes, so
-        // `UnexistedClassSerializer.writeClassDef`
+        // `NonexistentClassSerializer.writeClassDef`
         // can overwrite written classinfo and replace with real classinfo.
         short classId =
-            Objects.requireNonNull(classInfoMap.get(UnexistedMetaSharedClass.class)).classId;
+            Objects.requireNonNull(classInfoMap.get(NonexistentMetaShared.class)).classId;
         Preconditions.checkArgument(classId > 63 && classId < 8192, classId);
       } else {
-        register(UnexistedSkipClass.class);
+        register(NonexistentSkip.class);
       }
     }
   }
@@ -415,7 +415,14 @@ public class ClassResolver {
   public void register(Class<?> cls, boolean createSerializer) {
     register(cls);
     if (createSerializer) {
-      getSerializer(cls);
+      ClassInfo classInfo = getClassInfo(cls);
+      if (metaContextShareEnabled && needToWriteClassDef(classInfo.serializer)) {
+        ClassDef classDef = classInfo.classDef;
+        if (classDef == null) {
+          classDef = buildClassDef(classInfo);
+        }
+        buildMetaSharedClassInfo(Tuple2.of(classDef, null), classDef);
+      }
     }
   }
 
@@ -516,12 +523,20 @@ public class ClassResolver {
    * non-final to write class def, so that it can be deserialized by the peer still.
    */
   public boolean isMonomorphic(Class<?> clz) {
-    if (fury.getConfig().shareMetaContext()) {
+    if (fury.getConfig().isMetaShareEnabled()) {
       // can't create final map/collection type using TypeUtils.mapOf(TypeToken<K>,
       // TypeToken<V>)
-      return ReflectionUtils.isMonomorphic(clz)
-          && isInnerClass(clz)
-          && (!Map.class.isAssignableFrom(clz) && !Collection.class.isAssignableFrom(clz));
+      if (!ReflectionUtils.isMonomorphic(clz)) {
+        return false;
+      }
+      if (Map.class.isAssignableFrom(clz) || Collection.class.isAssignableFrom(clz)) {
+        return false;
+      }
+      if (clz.isArray()) {
+        Class<?> component = TypeUtils.getArrayComponent(clz);
+        return isMonomorphic(component);
+      }
+      return (isInnerClass(clz) || clz.isEnum());
     }
     return ReflectionUtils.isMonomorphic(clz);
   }
@@ -641,7 +656,7 @@ public class ClassResolver {
       Class<?> cls = entry.getKey();
       if (cls.getName().equals(className)) {
         LOG.info("Clear serializer for class {}.", className);
-        entry.getValue().serializer = Serializers.newSerializer(fury, cls, serializer);
+        entry.getValue().setSerializer(this, Serializers.newSerializer(fury, cls, serializer));
         classInfoCache = NIL_CLASS_INFO;
         return;
       }
@@ -658,7 +673,7 @@ public class ClassResolver {
       }
       if (className.startsWith(classNamePrefix)) {
         LOG.info("Clear serializer for class {}.", className);
-        entry.getValue().serializer = Serializers.newSerializer(fury, cls, serializer);
+        entry.getValue().setSerializer(this, Serializers.newSerializer(fury, cls, serializer));
         classInfoCache = NIL_CLASS_INFO;
       }
     }
@@ -682,9 +697,9 @@ public class ClassResolver {
 
   /**
    * Set serializer to avoid circular error when there is a serializer query for fields by {@link
-   * #getClassInfo} and {@link #getSerializer(Class)} which access current creating serializer. This
-   * method is used to avoid overwriting existing serializer for class when creating a data
-   * serializer for serialization of parts fields of a class.
+   * #buildMetaSharedClassInfo} and {@link #getSerializer(Class)} which access current creating
+   * serializer. This method is used to avoid overwriting existing serializer for class when
+   * creating a data serializer for serialization of parts fields of a class.
    */
   public <T> void setSerializerIfAbsent(Class<T> cls, Serializer<T> serializer) {
     Serializer<T> s = getSerializer(cls, false);
@@ -697,7 +712,7 @@ public class ClassResolver {
   public void clearSerializer(Class<?> cls) {
     ClassInfo classInfo = classInfoMap.get(cls);
     if (classInfo != null) {
-      classInfo.serializer = null;
+      classInfo.setSerializer(this, null);
     }
   }
 
@@ -742,7 +757,7 @@ public class ClassResolver {
     }
 
     // 2. Set `Serializer` for `ClassInfo`.
-    classInfo.serializer = serializer;
+    classInfo.setSerializer(this, serializer);
   }
 
   @SuppressWarnings("unchecked")
@@ -807,7 +822,12 @@ public class ClassResolver {
       // serialized, which will create a class info with serializer null, see `#writeClassInternal`
       return classInfo.serializer.getClass();
     } else {
-      if (cls.isEnum()) {
+      if (NonexistentClass.isNonexistent(cls)) {
+        return NonexistentClassSerializers.getSerializer(fury, "Unknown", cls).getClass();
+      }
+      if (cls.isArray()) {
+        return ArraySerializers.ObjectArraySerializer.class;
+      } else if (cls.isEnum()) {
         return EnumSerializer.class;
       } else if (Enum.class.isAssignableFrom(cls) && cls != Enum.class) {
         // handles an enum value that is an inner class. Eg: enum A {b{}};
@@ -816,9 +836,6 @@ public class ClassResolver {
         return CollectionSerializers.EnumSetSerializer.class;
       } else if (Charset.class.isAssignableFrom(cls)) {
         return Serializers.CharsetSerializer.class;
-      } else if (cls.isArray()) {
-        Preconditions.checkArgument(!cls.getComponentType().isPrimitive());
-        return ArraySerializers.ObjectArraySerializer.class;
       } else if (Functions.isLambda(cls)) {
         return LambdaSerializer.class;
       } else if (ReflectionUtils.isJdkProxy(cls)) {
@@ -832,6 +849,9 @@ public class ClassResolver {
       } else if (ByteBuffer.class.isAssignableFrom(cls)) {
         return BufferSerializers.ByteBufferSerializer.class;
       }
+      if (shimDispatcher.contains(cls)) {
+        return shimDispatcher.getSerializer(cls).getClass();
+      }
       if (fury.getConfig().checkJdkClassSerializable()) {
         if (cls.getName().startsWith("java") && !(Serializable.class.isAssignableFrom(cls))) {
           throw new UnsupportedOperationException(
@@ -985,7 +1005,9 @@ public class ClassResolver {
         }
       }
     } else {
-      LOG.info("Object of type {} can't be serialized by jit", cls);
+      if (fury.getConfig().isCodeGenEnabled()) {
+        LOG.info("Object of type {} can't be serialized by jit", cls);
+      }
       switch (fury.getCompatibleMode()) {
         case SCHEMA_CONSISTENT:
           return ObjectSerializer.class;
@@ -1241,155 +1263,162 @@ public class ClassResolver {
 
   /** Write classname for java serialization. */
   public void writeClass(MemoryBuffer buffer, ClassInfo classInfo) {
-    if (classInfo.classId == NO_CLASS_ID) { // no class id provided.
-      // use classname
-      if (metaContextShareEnabled) {
-        buffer.writeByte(USE_CLASS_VALUE_FLAG);
-        // FIXME(chaokunyang) Register class but not register serializer can't be used with
-        //  meta share mode, because no class def are sent to peer.
-        writeClassWithMetaShare(buffer, classInfo);
-      } else {
+    if (metaContextShareEnabled) {
+      // FIXME(chaokunyang) Register class but not register serializer can't be used with
+      //  meta share mode, because no class def are sent to peer.
+      writeClassWithMetaShare(buffer, classInfo);
+    } else {
+      if (classInfo.classId == NO_CLASS_ID) { // no class id provided.
+        // use classname
         // if it's null, it's a bug.
         assert classInfo.packageNameBytes != null;
         metaStringResolver.writeMetaStringBytesWithFlag(buffer, classInfo.packageNameBytes);
         assert classInfo.classNameBytes != null;
         metaStringResolver.writeMetaStringBytes(buffer, classInfo.classNameBytes);
+      } else {
+        // use classId
+        buffer.writeVarUint32(classInfo.classId << 1);
       }
-    } else {
-      // use classId
-      buffer.writeVarUint32(classInfo.classId << 1);
     }
   }
 
   public void writeClassWithMetaShare(MemoryBuffer buffer, ClassInfo classInfo) {
+    if (classInfo.classId != NO_CLASS_ID && !classInfo.needToWriteClassDef) {
+      buffer.writeVarUint32(classInfo.classId << 1);
+      return;
+    }
     MetaContext metaContext = fury.getSerializationContext().getMetaContext();
-    Preconditions.checkNotNull(
-        metaContext,
-        "Meta context must be set before serialization,"
-            + " please set meta context by SerializationContext.setMetaContext");
+    assert metaContext != null : SET_META__CONTEXT_MSG;
     IdentityObjectIntMap<Class<?>> classMap = metaContext.classMap;
     int newId = classMap.size;
     int id = classMap.putOrGet(classInfo.cls, newId);
     if (id >= 0) {
-      buffer.writeVarUint32(id);
+      buffer.writeVarUint32(id << 1 | 0b1);
     } else {
-      buffer.writeVarUint32(newId);
-      ClassDef classDef;
-      Serializer<?> serializer = classInfo.serializer;
-      Preconditions.checkArgument(serializer.getClass() != UnexistedClassSerializer.class);
-      if (fury.getConfig().getCompatibleMode() == CompatibleMode.COMPATIBLE
-          && (serializer instanceof Generated.GeneratedObjectSerializer
-              // May already switched to MetaSharedSerializer when update class info cache.
-              || serializer instanceof Generated.GeneratedMetaSharedSerializer
-              || serializer instanceof LazyInitBeanSerializer
-              || serializer instanceof ObjectSerializer
-              || serializer instanceof MetaSharedSerializer)) {
-        classDef =
-            classDefMap.computeIfAbsent(classInfo.cls, cls -> ClassDef.buildClassDef(fury, cls));
-      } else {
-        // TODO(chaokunyang) support more types meta-share serialization
-        classDef =
-            classDefMap.computeIfAbsent(
-                classInfo.cls,
-                cls ->
-                    ClassDef.buildClassDef(
-                        this,
-                        cls,
-                        new ArrayList<>(),
-                        ofHashMap(META_SHARE_FIELDS_INFO_KEY, "false")));
+      buffer.writeVarUint32(newId << 1 | 0b1);
+      ClassDef classDef = classInfo.classDef;
+      if (classDef == null) {
+        classDef = buildClassDef(classInfo);
       }
       metaContext.writingClassDefs.add(classDef);
     }
   }
 
-  private Class<?> readClassWithMetaShare(MemoryBuffer buffer) {
-    MetaContext metaContext = fury.getSerializationContext().getMetaContext();
-    Preconditions.checkNotNull(
-        metaContext,
-        "Meta context must be set before serialization,"
-            + " please set meta context by SerializationContext.setMetaContext");
-    int id = buffer.readVarUint32Small14();
-    List<ClassInfo> readClassInfos = metaContext.readClassInfos;
-    ClassInfo classInfo = readClassInfos.get(id);
-    if (classInfo == null) {
-      List<ClassDef> readClassDefs = metaContext.readClassDefs;
-      ClassDef classDef = readClassDefs.get(id);
-      Class<?> cls = loadClass(classDef.getClassName());
-      classInfo = getClassInfo(cls, false);
-      if (classInfo == null) {
-        Short classId = extRegistry.registeredClassIdMap.get(cls);
-        classInfo = new ClassInfo(this, cls, null, null, classId == null ? NO_CLASS_ID : classId);
-        classInfoMap.put(cls, classInfo);
-      }
-      readClassInfos.set(id, classInfo);
+  private ClassDef buildClassDef(ClassInfo classInfo) {
+    ClassDef classDef;
+    Serializer<?> serializer = classInfo.serializer;
+    Preconditions.checkArgument(serializer.getClass() != NonexistentClassSerializer.class);
+    if (needToWriteClassDef(serializer)) {
+      classDef =
+          classDefMap.computeIfAbsent(classInfo.cls, cls -> ClassDef.buildClassDef(fury, cls));
+    } else {
+      // Some type will use other serializers such MapSerializer and so on.
+      classDef =
+          classDefMap.computeIfAbsent(
+              classInfo.cls, cls -> ClassDef.buildClassDef(this, cls, new ArrayList<>(), false));
     }
-    return classInfo.cls;
+    classInfo.classDef = classDef;
+    return classDef;
+  }
+
+  boolean needToWriteClassDef(Serializer serializer) {
+    return fury.getConfig().getCompatibleMode() == CompatibleMode.COMPATIBLE
+        && (serializer instanceof Generated.GeneratedObjectSerializer
+            // May already switched to MetaSharedSerializer when update class info cache.
+            || serializer instanceof Generated.GeneratedMetaSharedSerializer
+            || serializer instanceof LazyInitBeanSerializer
+            || serializer instanceof ObjectSerializer
+            || serializer instanceof MetaSharedSerializer);
   }
 
   private ClassInfo readClassInfoWithMetaShare(MemoryBuffer buffer, MetaContext metaContext) {
-    Preconditions.checkNotNull(
-        metaContext,
-        "Meta context must be set before serialization,"
-            + " please set meta context by SerializationContext.setMetaContext");
-    int id = buffer.readVarUint32Small14();
-    List<ClassInfo> readClassInfos = metaContext.readClassInfos;
+    assert metaContext != null : SET_META__CONTEXT_MSG;
+    int header = buffer.readVarUint32Small14();
+    int id = header >>> 1;
+    if ((header & 0b1) == 0) {
+      return getOrUpdateClassInfo((short) id);
+    }
+    ObjectArray<ClassInfo> readClassInfos = metaContext.readClassInfos;
     ClassInfo classInfo = readClassInfos.get(id);
     if (classInfo == null) {
-      List<ClassDef> readClassDefs = metaContext.readClassDefs;
+      ObjectArray<ClassDef> readClassDefs = metaContext.readClassDefs;
       ClassDef classDef = readClassDefs.get(id);
-      if ("false".equals(classDef.getExtMeta().getOrDefault(META_SHARE_FIELDS_INFO_KEY, ""))) {
-        Class<?> cls = loadClass(classDef.getClassName());
-        classInfo = getClassInfo(cls);
+      Tuple2<ClassDef, ClassInfo> classDefTuple = extRegistry.classIdToDef.get(classDef.getId());
+      if (classDefTuple == null || classDefTuple.f1 == null) {
+        classInfo = buildMetaSharedClassInfo(classDefTuple, classDef);
       } else {
-        Tuple2<ClassDef, ClassInfo> classDefTuple = extRegistry.classIdToDef.get(classDef.getId());
-        if (classDefTuple == null || classDefTuple.f1 == null) {
-          if (classDefTuple != null) {
-            classDef = classDefTuple.f0;
-          }
-          Class<?> cls = loadClass(classDef.getClassName());
-          classInfo = getMetaSharedClassInfo(classDef, cls);
-          // Share serializer for same version class def to avoid too much different meta
-          // context take up too much memory.
-          putClassDef(classDef, classInfo);
-        } else {
-          classInfo = classDefTuple.f1;
-        }
+        classInfo = classDefTuple.f1;
       }
       readClassInfos.set(id, classInfo);
     }
     return classInfo;
   }
 
+  private ClassInfo buildMetaSharedClassInfo(
+      Tuple2<ClassDef, ClassInfo> classDefTuple, ClassDef classDef) {
+    ClassInfo classInfo;
+    if (classDefTuple != null) {
+      classDef = classDefTuple.f0;
+    }
+    Class<?> cls = loadClass(classDef.getClassSpec());
+    if (!classDef.isObjectType()) {
+      classInfo = getClassInfo(cls);
+    } else {
+      classInfo = getMetaSharedClassInfo(classDef, cls);
+    }
+    // Share serializer for same version class def to avoid too much different meta
+    // context take up too much memory.
+    putClassDef(classDef, classInfo);
+    return classInfo;
+  }
+
   // TODO(chaokunyang) if ClassDef is consistent with class in this process,
   //  use existing serializer instead.
   private ClassInfo getMetaSharedClassInfo(ClassDef classDef, Class<?> clz) {
-    if (clz == UnexistedSkipClass.class) {
-      clz = UnexistedMetaSharedClass.class;
+    if (clz == NonexistentSkip.class) {
+      clz = NonexistentMetaShared.class;
     }
     Class<?> cls = clz;
     Short classId = extRegistry.registeredClassIdMap.get(cls);
     ClassInfo classInfo =
         new ClassInfo(this, cls, null, null, classId == null ? NO_CLASS_ID : classId);
-    if (cls == UnexistedMetaSharedClass.class) {
-      classInfo.serializer = new UnexistedClassSerializer(fury, classDef);
-      // ensure `UnexistedMetaSharedClass` registered to write fixed-length class def,
-      // so we can rewrite it in `UnexistedClassSerializer`.
-      Preconditions.checkNotNull(classId);
+    if (NonexistentClass.class.isAssignableFrom(TypeUtils.getComponentIfArray(cls))) {
+      if (cls == NonexistentMetaShared.class) {
+        classInfo.setSerializer(this, new NonexistentClassSerializer(fury, classDef));
+        // ensure `NonexistentMetaSharedClass` registered to write fixed-length class def,
+        // so we can rewrite it in `NonexistentClassSerializer`.
+        Preconditions.checkNotNull(classId);
+      } else {
+        classInfo.serializer =
+            NonexistentClassSerializers.getSerializer(fury, classDef.getClassName(), cls);
+      }
       return classInfo;
     }
     if (clz.isArray() || cls.isEnum()) {
       return getClassInfo(cls);
     }
     Class<? extends Serializer> sc =
-        fury.getJITContext()
-            .registerSerializerJITCallback(
-                () -> MetaSharedSerializer.class,
-                () -> CodecUtils.loadOrGenMetaSharedCodecClass(fury, cls, classDef),
-                c -> classInfo.serializer = Serializers.newSerializer(fury, cls, c));
+        getMetaSharedDeserializerClassFromGraalvmRegistry(cls, classDef);
+    if (sc == null) {
+      if (GraalvmSupport.isGraalRuntime()) {
+        sc = MetaSharedSerializer.class;
+        LOG.warn(
+            "Can't generate class at runtime in graalvm for class def {}, use {} instead",
+            classDef,
+            sc);
+      } else {
+        sc =
+            fury.getJITContext()
+                .registerSerializerJITCallback(
+                    () -> MetaSharedSerializer.class,
+                    () -> CodecUtils.loadOrGenMetaSharedCodecClass(fury, cls, classDef),
+                    c -> classInfo.setSerializer(this, Serializers.newSerializer(fury, cls, c)));
+      }
+    }
     if (sc == MetaSharedSerializer.class) {
-      classInfo.serializer = new MetaSharedSerializer(fury, cls, classDef);
+      classInfo.setSerializer(this, new MetaSharedSerializer(fury, cls, classDef));
     } else {
-      classInfo.serializer = Serializers.newSerializer(fury, cls, sc);
+      classInfo.setSerializer(this, Serializers.newSerializer(fury, cls, sc));
     }
     return classInfo;
   }
@@ -1401,11 +1430,31 @@ public class ClassResolver {
    */
   public void writeClassDefs(MemoryBuffer buffer) {
     MetaContext metaContext = fury.getSerializationContext().getMetaContext();
-    buffer.writeVarUint32Small7(metaContext.writingClassDefs.size());
-    for (ClassDef classDef : metaContext.writingClassDefs) {
-      classDef.writeClassDef(buffer);
+    ObjectArray<ClassDef> writingClassDefs = metaContext.writingClassDefs;
+    final int size = writingClassDefs.size;
+    buffer.writeVarUint32Small7(size);
+    if (buffer.isHeapFullyWriteable()) {
+      writeClassDefs(buffer, writingClassDefs, size);
+    } else {
+      for (int i = 0; i < size; i++) {
+        writingClassDefs.get(i).writeClassDef(buffer);
+      }
     }
-    metaContext.writingClassDefs.clear();
+    metaContext.writingClassDefs.size = 0;
+  }
+
+  private void writeClassDefs(
+      MemoryBuffer buffer, ObjectArray<ClassDef> writingClassDefs, int size) {
+    int writerIndex = buffer.writerIndex();
+    for (int i = 0; i < size; i++) {
+      byte[] encoded = writingClassDefs.get(i).getEncoded();
+      int bytesLen = encoded.length;
+      buffer.ensure(writerIndex + bytesLen);
+      final byte[] targetArray = buffer.getHeapMemory();
+      System.arraycopy(encoded, 0, targetArray, writerIndex, bytesLen);
+      writerIndex += bytesLen;
+    }
+    buffer.writerIndex(writerIndex);
   }
 
   /**
@@ -1421,8 +1470,7 @@ public class ClassResolver {
     int numClassDefs = buffer.readVarUint32Small14();
     for (int i = 0; i < numClassDefs; i++) {
       long id = buffer.readInt64();
-      long hash = id >>> 8;
-      Tuple2<ClassDef, ClassInfo> tuple2 = extRegistry.classIdToDef.get(hash);
+      Tuple2<ClassDef, ClassInfo> tuple2 = extRegistry.classIdToDef.get(id);
       if (tuple2 != null) {
         int size =
             (id & SIZE_TWO_BYTES_FLAG) == 0
@@ -1515,7 +1563,12 @@ public class ClassResolver {
       // ReplaceResolveSerializer.ReplaceStub
       classInfo.classId = NO_CLASS_ID;
     }
-    writeClass(buffer, classInfo);
+    if (classInfo.classId != NO_CLASS_ID) {
+      buffer.writeVarUint32(classInfo.classId << 1);
+    } else {
+      metaStringResolver.writeMetaStringBytesWithFlag(buffer, classInfo.packageNameBytes);
+      metaStringResolver.writeMetaStringBytes(buffer, classInfo.classNameBytes);
+    }
     classInfo.classId = classId;
   }
 
@@ -1528,9 +1581,6 @@ public class ClassResolver {
     int header = buffer.readVarUint32Small14();
     final ClassInfo classInfo;
     if ((header & 0b1) != 0) {
-      if (metaContextShareEnabled) {
-        return readClassWithMetaShare(buffer);
-      }
       MetaStringBytes packageBytes = metaStringResolver.readMetaStringBytesWithFlag(buffer, header);
       MetaStringBytes simpleClassNameBytes = metaStringResolver.readMetaStringBytes(buffer);
       classInfo = loadBytesToClassInfo(packageBytes, simpleClassNameBytes);
@@ -1547,23 +1597,19 @@ public class ClassResolver {
    * ClassInfo)} is faster since it use a non-global class info cache.
    */
   public ClassInfo readClassInfo(MemoryBuffer buffer) {
-    int header = buffer.readVarUint32Small14();
-    if ((header & 0b1) != 0) {
-      ClassInfo classInfo;
-      if (metaContextShareEnabled) {
-        classInfo =
-            readClassInfoWithMetaShare(buffer, fury.getSerializationContext().getMetaContext());
-      } else {
-        classInfo = readClassInfoFromBytes(buffer, classInfoCache, header);
-      }
-      classInfoCache = classInfo;
-      currentReadClass = classInfo.cls;
-      return classInfo;
-    } else {
-      ClassInfo classInfo = getOrUpdateClassInfo((short) (header >> 1));
-      currentReadClass = classInfo.cls;
-      return classInfo;
+    if (metaContextShareEnabled) {
+      return readClassInfoWithMetaShare(buffer, fury.getSerializationContext().getMetaContext());
     }
+    int header = buffer.readVarUint32Small14();
+    ClassInfo classInfo;
+    if ((header & 0b1) != 0) {
+      classInfo = readClassInfoFromBytes(buffer, classInfoCache, header);
+      classInfoCache = classInfo;
+    } else {
+      classInfo = getOrUpdateClassInfo((short) (header >> 1));
+    }
+    currentReadClass = classInfo.cls;
+    return classInfo;
   }
 
   /**
@@ -1572,6 +1618,9 @@ public class ClassResolver {
    */
   @CodegenInvoke
   public ClassInfo readClassInfo(MemoryBuffer buffer, ClassInfo classInfoCache) {
+    if (metaContextShareEnabled) {
+      return readClassInfoWithMetaShare(buffer, fury.getSerializationContext().getMetaContext());
+    }
     int header = buffer.readVarUint32Small14();
     if ((header & 0b1) != 0) {
       return readClassInfoByCache(buffer, classInfoCache, header);
@@ -1583,6 +1632,9 @@ public class ClassResolver {
   /** Read class info, update classInfoHolder if cache not hit. */
   @CodegenInvoke
   public ClassInfo readClassInfo(MemoryBuffer buffer, ClassInfoHolder classInfoHolder) {
+    if (metaContextShareEnabled) {
+      return readClassInfoWithMetaShare(buffer, fury.getSerializationContext().getMetaContext());
+    }
     int header = buffer.readVarUint32Small14();
     if ((header & 0b1) != 0) {
       return readClassInfoFromBytes(buffer, classInfoHolder, header);
@@ -1652,35 +1704,12 @@ public class ClassResolver {
       MetaStringBytes packageBytes,
       MetaStringBytes simpleClassNameBytes) {
     String packageName = packageBytes.decode(PACKAGE_DECODER);
-    String rawPkg = packageName;
     String className = simpleClassNameBytes.decode(TYPE_NAME_DECODER);
-    boolean isArray = className.startsWith(ClassInfo.ARRAY_PREFIX);
-    if (isArray) {
-      int dimension = 0;
-      while (className.charAt(dimension) == ClassInfo.ARRAY_PREFIX.charAt(0)) {
-        dimension++;
-      }
-      packageName = StringUtils.repeat("[", dimension) + "L" + packageName;
-      className = className.substring(dimension) + ";";
-    }
-    boolean isEnum = className.startsWith(ClassInfo.ENUM_PREFIX);
-    if (isEnum) {
-      className = className.substring(1);
-    }
-    String entireClassName;
-    if (StringUtils.isBlank(rawPkg)) {
-      if (isArray) {
-        entireClassName = packageName + className;
-      } else {
-        entireClassName = className;
-      }
-    } else {
-      entireClassName = packageName + "." + className;
-    }
+    ClassSpec classSpec = Encoders.decodePkgAndClass(packageName, className);
     MetaStringBytes fullClassNameBytes =
         metaStringResolver.getOrCreateMetaStringBytes(
-            PACKAGE_ENCODER.encode(entireClassName, MetaString.Encoding.UTF_8));
-    Class<?> cls = loadClass(entireClassName, isArray, isEnum);
+            PACKAGE_ENCODER.encode(classSpec.entireClassName, MetaString.Encoding.UTF_8));
+    Class<?> cls = loadClass(classSpec.entireClassName, classSpec.isEnum, classSpec.dimension);
     ClassInfo classInfo =
         new ClassInfo(
             cls,
@@ -1691,12 +1720,9 @@ public class ClassResolver {
             null,
             null,
             NO_CLASS_ID);
-    if (cls == UnexistedSkipEnumClass.class) {
-      classInfo.serializer = new UnexistedEnumClassSerializer(fury);
-    } else if (cls == UnexistedArrayClass.class) {
-      classInfo.serializer = new UnexistedArrayClassSerializer(fury, entireClassName);
-    } else if (cls == UnexistedEnumArrayClass.class) {
-      classInfo.serializer = new UnexistedEnumArrayClassSerializer(fury, entireClassName);
+    if (NonexistentClass.class.isAssignableFrom(TypeUtils.getComponentIfArray(cls))) {
+      classInfo.serializer =
+          NonexistentClassSerializers.getSerializer(fury, classSpec.entireClassName, cls);
     } else {
       // don't create serializer here, if the class is an interface,
       // there won't be serializer since interface has no instance.
@@ -1721,7 +1747,7 @@ public class ClassResolver {
     Class<?> cls = classNameBytes2Class.get(byteString);
     if (cls == null) {
       Preconditions.checkNotNull(byteString);
-      String className = byteString.decode('.', '_');
+      String className = byteString.decode(Encoders.GENERIC_DECODER);
       cls = loadClass(className);
       classNameBytes2Class.put(byteString, cls);
     }
@@ -1738,10 +1764,14 @@ public class ClassResolver {
   }
 
   private Class<?> loadClass(String className) {
-    return loadClass(className, false, false);
+    return loadClass(className, false, 0);
   }
 
-  private Class<?> loadClass(String className, boolean isArray, boolean isEnum) {
+  private Class<?> loadClass(ClassSpec classSpec) {
+    return loadClass(classSpec.entireClassName, classSpec.isEnum, classSpec.dimension);
+  }
+
+  private Class<?> loadClass(String className, boolean isEnum, int arrayDims) {
     extRegistry.classChecker.checkClass(this, className);
     try {
       return Class.forName(className, false, fury.getClassLoader());
@@ -1753,16 +1783,10 @@ public class ClassResolver {
             String.format(
                 "Class %s not found from classloaders [%s, %s]",
                 className, fury.getClassLoader(), Thread.currentThread().getContextClassLoader());
-        if (fury.getConfig().deserializeUnexistedClass()) {
+        if (fury.getConfig().deserializeNonexistentClass()) {
           LOG.warn(msg);
-          if (isArray) {
-            return isEnum ? UnexistedEnumArrayClass.class : UnexistedArrayClass.class;
-          } else if (isEnum) {
-            return UnexistedSkipEnumClass.class;
-          } else {
-            // FIXME create a subclass dynamically may be better?
-            return UnexistedSkipClass.class;
-          }
+          return NonexistentClass.getNonexistentClass(
+              className, isEnum, arrayDims, metaContextShareEnabled);
         }
         throw new IllegalStateException(msg, ex);
       }
@@ -1913,4 +1937,28 @@ public class ClassResolver {
     }
     return null;
   }
+
+  private Class<? extends Serializer> getMetaSharedDeserializerClassFromGraalvmRegistry(
+      Class<?> cls, ClassDef classDef) {
+    List<ClassResolver> classResolvers = GRAALVM_REGISTRY.get(fury.getConfig().getConfigHash());
+    if (classResolvers == null || classResolvers.isEmpty()) {
+      return null;
+    }
+    for (ClassResolver classResolver : classResolvers) {
+      if (classResolver != this) {
+        Tuple2<ClassDef, ClassInfo> tuple2 =
+            classResolver.extRegistry.classIdToDef.get(classDef.getId());
+        if (tuple2 != null && tuple2.f1 != null) {
+          return tuple2.f1.serializer.getClass();
+        }
+      }
+    }
+    if (GraalvmSupport.isGraalRuntime()) {
+      if (Functions.isLambda(cls) || ReflectionUtils.isJdkProxy(cls)) {
+        return null;
+      }
+      throw new RuntimeException(String.format("Class %s is not registered", cls));
+    }
+    return null;
+  }
 }
