@@ -34,8 +34,8 @@ import org.apache.fury.resolver.ClassInfo;
 import org.apache.fury.resolver.ClassResolver;
 import org.apache.fury.resolver.FieldResolver;
 import org.apache.fury.resolver.RefResolver;
-import org.apache.fury.serializer.collection.CollectionSerializer;
-import org.apache.fury.serializer.collection.MapSerializer;
+import org.apache.fury.serializer.collection.AbstractCollectionSerializer;
+import org.apache.fury.serializer.collection.AbstractMapSerializer;
 import org.apache.fury.util.Preconditions;
 import org.apache.fury.util.record.RecordInfo;
 import org.apache.fury.util.record.RecordUtils;
@@ -241,9 +241,16 @@ public final class CompatibleSerializer<T> extends CompatibleSerializerBase<T> {
     // following write is consistent with `BaseSeqCodecBuilder.serializeForCollection`
     ClassInfo classInfo = fieldInfo.getClassInfo(fieldValue.getClass());
     classResolver.writeClass(buffer, classInfo);
-    CollectionSerializer collectionSerializer = (CollectionSerializer) classInfo.getSerializer();
-    collectionSerializer.setElementSerializer(elementClassInfo.getSerializer());
-    collectionSerializer.write(buffer, fieldValue);
+    AbstractCollectionSerializer collectionSerializer =
+        (AbstractCollectionSerializer) classInfo.getSerializer();
+    try {
+      collectionSerializer.setElementSerializer(elementClassInfo.getSerializer());
+      collectionSerializer.write(buffer, fieldValue);
+    } finally {
+      // Some collectionSerializer may overwrite write/read method, then clear element serializer
+      // may not got invoked.
+      collectionSerializer.setElementSerializer(null);
+    }
   }
 
   private void writeMapKVFinal(
@@ -255,10 +262,17 @@ public final class CompatibleSerializer<T> extends CompatibleSerializerBase<T> {
     // following write is consistent with `BaseSeqCodecBuilder.serializeForMap`
     ClassInfo classInfo = fieldInfo.getClassInfo(fieldValue.getClass());
     classResolver.writeClass(buffer, classInfo);
-    MapSerializer mapSerializer = (MapSerializer) classInfo.getSerializer();
-    mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
-    mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
-    mapSerializer.write(buffer, fieldValue);
+    AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
+    try {
+      mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
+      mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
+      mapSerializer.write(buffer, fieldValue);
+    } finally {
+      // Some mmapSerializer may overwrite write/read method, then clear serializer
+      // may not got invoked.
+      mapSerializer.setKeySerializer(null);
+      mapSerializer.setValueSerializer(null);
+    }
   }
 
   private void writeMapKeyFinal(
@@ -268,9 +282,15 @@ public final class CompatibleSerializer<T> extends CompatibleSerializerBase<T> {
     // following write is consistent with `BaseSeqCodecBuilder.serializeForMap`
     ClassInfo classInfo = fieldInfo.getClassInfo(fieldValue.getClass());
     classResolver.writeClass(buffer, classInfo);
-    MapSerializer mapSerializer = (MapSerializer) classInfo.getSerializer();
-    mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
-    mapSerializer.write(buffer, fieldValue);
+    AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
+    try {
+      mapSerializer.setKeySerializer(keyClassInfo.getSerializer());
+      mapSerializer.write(buffer, fieldValue);
+    } finally {
+      // Some mmapSerializer may overwrite write/read method, then clear serializer
+      // may not got invoked.
+      mapSerializer.setKeySerializer(null);
+    }
   }
 
   private void writeMapValueFinal(
@@ -280,9 +300,15 @@ public final class CompatibleSerializer<T> extends CompatibleSerializerBase<T> {
     // following write is consistent with `BaseSeqCodecBuilder.serializeForMap`
     ClassInfo classInfo = fieldInfo.getClassInfo(fieldValue.getClass());
     classResolver.writeClass(buffer, classInfo);
-    MapSerializer mapSerializer = (MapSerializer) classInfo.getSerializer();
-    mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
-    mapSerializer.write(buffer, fieldValue);
+    AbstractMapSerializer mapSerializer = (AbstractMapSerializer) classInfo.getSerializer();
+    try {
+      mapSerializer.setValueSerializer(valueClassInfo.getSerializer());
+      mapSerializer.write(buffer, fieldValue);
+    } finally {
+      // Some mmapSerializer may overwrite write/read method, then clear serializer
+      // may not got invoked.
+      mapSerializer.setValueSerializer(null);
+    }
   }
 
   @SuppressWarnings("unchecked")
