@@ -371,6 +371,13 @@ public class TypeUtils {
     return dimension;
   }
 
+  public static Class<?> getComponentIfArray(Class<?> type) {
+    if (type.isArray()) {
+      return getArrayComponent(type);
+    }
+    return type;
+  }
+
   public static Class<?> getArrayComponent(Class<?> type) {
     return getArrayComponentInfo(type).f0;
   }
@@ -394,7 +401,7 @@ public class TypeUtils {
   /** Returns s string that represents array type declaration of type. */
   public static String getArrayType(Class<?> type) {
     Tuple2<Class<?>, Integer> info = getArrayComponentInfo(type);
-    StringBuilder typeBuilder = new StringBuilder(ReflectionUtils.getCanonicalName(info.f0));
+    StringBuilder typeBuilder = new StringBuilder(ReflectionUtils.getLiteralName(info.f0));
     for (int i = 0; i < info.f1; i++) {
       typeBuilder.append("[]");
     }
@@ -403,7 +410,7 @@ public class TypeUtils {
 
   /** Create an array type declaration from elemType and dimensions. */
   public static String getArrayType(Class<?> elemType, int[] dimensions) {
-    StringBuilder typeBuilder = new StringBuilder(ReflectionUtils.getCanonicalName(elemType));
+    StringBuilder typeBuilder = new StringBuilder(ReflectionUtils.getLiteralName(elemType));
     for (int i = 0; i < dimensions.length; i++) {
       typeBuilder.append('[').append(dimensions[i]).append(']');
     }
@@ -711,4 +718,11 @@ public class TypeUtils {
 
     return new ArrayList<>(allTypeArguments);
   }
+
+  public static boolean isEnumArray(Class<?> clz) {
+    if (!clz.isArray()) {
+      return false;
+    }
+    return getArrayComponent(clz).isEnum();
+  }
 }
