@@ -53,9 +53,7 @@ where
         // ref flag
         let ref_flag = deserializer.reader.i8();
 
-        if ref_flag == (RefFlag::NotNullValueFlag as i8)
-            || ref_flag == (RefFlag::RefValueFlag as i8)
-        {
+        if ref_flag == (RefFlag::NotNullValue as i8) || ref_flag == (RefFlag::RefValue as i8) {
             // type_id
             let type_id = deserializer.reader.i16();
             let ty = if Self::is_vec() {
@@ -71,9 +69,9 @@ where
             } else {
                 Ok(Self::read(deserializer)?)
             }
-        } else if ref_flag == (RefFlag::NullFlag as i8) {
+        } else if ref_flag == (RefFlag::Null as i8) {
             Err(Error::Null)
-        } else if ref_flag == (RefFlag::RefFlag as i8) {
+        } else if ref_flag == (RefFlag::Ref as i8) {
             Err(Error::Ref)
         } else {
             Err(Error::BadRefFlag)
@@ -166,7 +164,7 @@ impl<T: Deserialize + Eq + std::hash::Hash> Deserialize for HashSet<T> {
 impl Deserialize for NaiveDateTime {
     fn read(deserializer: &mut DeserializerState) -> Result<Self, Error> {
         let timestamp = deserializer.reader.u64();
-        let ret = NaiveDateTime::from_timestamp_millis(timestamp as i64);
+        let ret = DateTime::from_timestamp_millis(timestamp as i64).map(|dt| dt.naive_utc());
         match ret {
             Some(r) => Ok(r),
             None => Err(Error::NaiveDateTime),
@@ -189,9 +187,7 @@ impl<T: Deserialize> Deserialize for Option<T> {
         // ref flag
         let ref_flag = deserializer.reader.i8();
 
-        if ref_flag == (RefFlag::NotNullValueFlag as i8)
-            || ref_flag == (RefFlag::RefValueFlag as i8)
-        {
+        if ref_flag == (RefFlag::NotNullValue as i8) || ref_flag == (RefFlag::RefValue as i8) {
             // type_id
             let type_id = deserializer.reader.i16();
 
@@ -203,9 +199,9 @@ impl<T: Deserialize> Deserialize for Option<T> {
             } else {
                 Ok(Self::read(deserializer)?)
             }
-        } else if ref_flag == (RefFlag::NullFlag as i8) {
+        } else if ref_flag == (RefFlag::Null as i8) {
             Ok(None)
-        } else if ref_flag == (RefFlag::RefFlag as i8) {
+        } else if ref_flag == (RefFlag::Ref as i8) {
             Err(Error::Ref)
         } else {
             Err(Error::BadRefFlag)
